// Example of how to read a barcode from a cv::Mat image
//

#include <iostream>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/imgcodecs.hpp>

#include "SoftekBarcodeDLL.h"

using namespace cv;

static void convertBitmapData(unsigned char* in, BITMAP* pBm)
{
	int x, y;
	unsigned char* out;

	out = (unsigned char*)pBm->bmBits + (pBm->bmWidthBytes * pBm->bmHeight);

	for (y = 0; y < pBm->bmHeight; y++)
	{
		out -= pBm->bmWidthBytes;
		for (x = 0; x < pBm->bmWidth; x++)
		{
			out[x] = in[x];
		}
		in += pBm->bmWidth;
	}
}

int main()
{

	int i;
	int n;
	BITMAP bm;
	HANDLE hBarcode;

    // Load the file into a Mat
	Mat img = imread("file.jpg", IMREAD_GRAYSCALE);

	if (img.empty())
	{
		std::cout << "Could not read the image: " << std::endl;
	}

	std::cout << "Width of image is " << img.size().width << std::endl;
	std::cout << "Height of image is " << img.size().height << std::endl;

    // Create th BITMAP strcuture
	bm.bmWidth = img.size().width;
	bm.bmHeight = img.size().height;
	bm.bmBitsPixel = 8; // from IMREAD_GRAYSCALE
	bm.bmWidthBytes = bm.bmWidth;
	if (bm.bmWidthBytes % 4) bm.bmWidthBytes += 4 - (bm.bmWidthBytes % 4); // Pad to 4 byte boundary
	bm.bmPlanes = 1;
	bm.bmType = 1;
	bm.bmBits = (void*)malloc(bm.bmWidthBytes * bm.bmHeight);

    // Copy the bitmap data to the bitmap strcuture
	convertBitmapData((unsigned char*)img.data, &bm);

    // Create an instance of the barcode sdk and set some properties
	hBarcode = mtCreateBarcodeInstance();

	mtSetReadDataMatrix(hBarcode, 1);
    
    // Use mtScanBarCodeFromBitmapA to read from the parts of BITMAP structure
    n = mtScanBarCodeFromBitmapA(hBarcode, bm.bmType, bm.bmWidth, bm.bmHeight, bm.bmWidthBytes, bm.bmPlanes, bm.bmBitsPixel, bm.bmBits);

    // Or create a HBITMAP and use mtScanBarCodeFromBitmap
    // HBITMAP hBitmap = CreateBitmapIndirect(&bm);
    // n = mtScanBarCodeFromBitmap(hBarcode, hBitmap);
    // DeleteObject(hBitmap);


	std::cout << "Number of barcodes is " << n << std::endl;

    // Get the results
	for (i = 1; i <= n; i++) {
		std::cout << "Value: " << mtGetBarString(hBarcode, i) << std::endl;
		std::cout << "Type: " << mtGetBarStringType(hBarcode, i) << std::endl;
	}
}

